@@ -1,6 +1,6 @@
 ##===========================================================================
 ##
-## Copyright (c) 2025 Marco Colombo
+## Copyright (c) 2025-2026 Marco Colombo
 ##
 ## This program is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -21,6 +21,9 @@
 #'
 #' Reports some summary statistics from the results of a run of [fuzz].
 #'
+#' The use of unicode icons in the output messages can be disabled by setting
+#' `options(cli.unicode = FALSE)`.
+#'
 #' @param object An object of class `cbtf`.
 #' @param tabulate Whether a tabulation of results should be printed out
 #'        (`TRUE` by default). The tabulation can always be retrieved from
@@ -50,6 +53,7 @@
 #'
 #' @export
 summary.cbtf <- function(object, tabulate = TRUE, ...) {
+  validate_class(tabulate, "logical", scalar = TRUE, from = "summary")
   df <- cbind(do.call(rbind, object$runs),
               fun = rep(object$funs, length(object$runs)),
               what = sapply(object$runs, function(x) attr(x, "what")))
@@ -66,14 +70,15 @@ summary.cbtf <- function(object, tabulate = TRUE, ...) {
 
 #' Print the results from a fuzz run
 #'
-#' This formats with colours the results from a fuzz run and prints them to
+#' This formats the results from a fuzz run with colours and prints them to
 #' the terminal.
 #'
+#' The use of unicode icons in the output messages can be disabled by setting
+#' `options(cli.unicode = FALSE)`.
+#'
 #' @param x An object of class `cbtf`.
-#' @param show_all Whether all results should be printed. By default (`FALSE`),
-#'        only the functions that reported an error or a warning are printed.
-#'        If `TRUE`, all functions tested are printed, including those that
-#'        were successful or were skipped.
+#' @param show A character vector representing the subset of results be
+#'        printed, any of "fail", "warn", "skip", "ok" and "all".
 #' @param ... Further arguments passed to or from other methods.
 #'        These are currently ignored.
 #'
@@ -83,21 +88,26 @@ summary.cbtf <- function(object, tabulate = TRUE, ...) {
 #' @examples
 #' res <- fuzz(funs = c("list", "matrix", "mean"),
 #'             what = test_inputs(c("numeric", "raw")))
-#' print(res, show.all = TRUE)
+#' print(res)
+#' print(res, show = "all")
 #'
 #' @seealso [summary.cbtf]
 #'
 #' @export
-print.cbtf <- function(x, show_all = FALSE, ...) {
+print.cbtf <- function(x, show = c("fail", "warn"), ...) {
+  validate_class(show, "character", from = "print")
+  show <- tolower(show)
+  if ("all" %in% show)
+    show <- c("fail", "warn", "skip", "ok")
   summary.stats <- compute_summary_stats(x)
   max.name <- max(c(0, nchar(x$funs))) + 1
   res.size <- nchar(tocolour("FAIL")) # include ANSI formatting, if present
-  for (run in x$runs) {
+  for (idx in seq_along(x$runs)) {
+    run <- x$runs[[idx]]
     run$fun <- x$funs
-    if (!show_all)
-      run <- run[run$res %in% c("FAIL", "WARN"), ]
+    run <- run[tolower(run$res) %in% show, ]
     if (nrow(run) > 0) {
-      cli::cli_h3(paste("Test input:", cli::style_bold(attributes(run)$what)))
+      cli::cli_h3("Test input [[{idx}]]: {.strong {attributes(run)$what}}")
       cat(sprintf("%*s  %*s  %s\n",
                   max.name, run$fun, res.size, tocolour(run$res), run$msg),
           sep = "")
@@ -106,6 +116,31 @@ print.cbtf <- function(x, show_all = FALSE, ...) {
   cat("\n", summary.stats, "\n")
 }
 
+#' Extract the results for a specific test input
+#'
+#' @param x An object of class `cbtf`.
+#' @param i An index between 1 and the number of test inputs used.
+#'
+#' @return
+#' If the index is valid, a data frame containing the following columns and
+#' attributes:
+#' \item{res}{One of "OK", "FAIL", "WARN" or "SKIP" for each combination of
+#'       function and input tested (see the *Value* section in [fuzz]).}
+#' \item{msg}{The message received in case of error, warning or skip,
+#'       or an empty string if no failure occurred.}
+#' \item{attr(*, "what")}{The character representation of the input tested.}
+#' Otherwise, `FALSE`.
+#'
+#' @examples
+#' res <- fuzz(funs = c("list", "matrix", "mean"),
+#'             what = test_inputs(c("numeric", "raw")))
+#' res[[6]]
+#'
+#' @export
+`[[.cbtf` <- function(x, i) {
+  i %in% seq_along(x$runs) && return(x$runs[[i]])
+}
+
 #' Compute the number of tests performed
 #'
 #' @param x An object of class `cbtf`.
