@@ -6,12 +6,12 @@ test_that("test_inputs", {
   expect_error(test_inputs(""),
                "'use' is an empty character")
   expect_error(test_inputs("error"),
-               "[test_inputs] No valid tests selected",
+               "[test_inputs] No valid tests selected, valid names are: 'all'",
                fixed = TRUE)
   expect_error(test_inputs(skip = NA),
                "'skip' should be of class character")
   expect_error(test_inputs("scalar", skip = "scalar"),
-               "No valid tests selected")
+               "No valid tests selected, valid names are: 'all', 'scalar'")
 
   expect_equal(test_inputs(use = "scalar", skip = "nonexisting"),
                test_inputs(use = "scalar"))
@@ -24,7 +24,7 @@ test_that("test_inputs", {
   expect_equal(test_inputs("help"),
                c("all", "scalar", "numeric", "integer", "logical",
                  "character", "factor", "data.frame", "matrix", "array",
-                 "date", "raw", "list"))
+                 "date", "raw", "na", "list"))
   expect_equal(test_inputs("help"),
                test_inputs(c("scalar", "help")))
 })
